﻿using FishFactoryContracts.BindingModels;
using FishFactoryContracts.SearchModels;
using FishFactoryContracts.StoragesContracts;
using FishFactoryContracts.ViewModels;
using FishFactoryFileImplement.Models;

namespace FishFactoryFileImplement.Implements
{
    public class CannedStorage : ICannedStorage
    {
        private readonly DataFileSingleton _source;
        public CannedStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        public List<CannedViewModel> GetFullList()
        {
            return _source.Canneds.Select(x => x.GetViewModel).ToList();
        }
        public List<CannedViewModel> GetFilteredList(CannedSearchModel model)
        {
            if (string.IsNullOrEmpty(model.CannedName))
            {
                return new();
            }
            return _source.Canneds.Where(x => x.CannedName.Contains(model.CannedName)).Select(x => x.GetViewModel).ToList();
        }
        public CannedViewModel? GetElement(CannedSearchModel model)
        {
            if (string.IsNullOrEmpty(model.CannedName) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Canneds.FirstOrDefault
                (x => (!string.IsNullOrEmpty(model.CannedName) && x.CannedName == model.CannedName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public CannedViewModel? Insert(CannedBindingModel model)
        {
            model.Id = _source.Canneds.Count > 0 ? _source.Canneds.Max(x => x.Id) + 1 : 1;
            var newCanned = Canned.Create(model);
            if (newCanned == null)
            {
                return null;
            }
            _source.Canneds.Add(newCanned);
            _source.SaveCanneds();
            return newCanned.GetViewModel;
        }
        public CannedViewModel? Update(CannedBindingModel model)
        {
            var canned = _source.Canneds.FirstOrDefault(x => x.Id == model.Id);
            if (canned == null)
            {
                return null;
            }
            canned.Update(model);
            _source.SaveCanneds();
            return canned.GetViewModel;
        }
        public CannedViewModel? Delete(CannedBindingModel model)
        {
            var element = _source.Canneds.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                _source.Canneds.Remove(element);
                _source.SaveCanneds();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
